@@ -16,7 +16,6 @@ package buflsp
 
 import (
 	"maps"
-	"strings"
 
 	"github.com/bufbuild/protocompile/experimental/ir"
 	"github.com/bufbuild/protocompile/experimental/seq"
@@ -24,7 +23,7 @@ import (
 	"github.com/bufbuild/protocompile/experimental/token/keyword"
 	"github.com/google/cel-go/cel"
 	"github.com/google/cel-go/common/ast"
-	"github.com/google/cel-go/common/operators"
+	"github.com/google/cel-go/common/overloads"
 	exprpb "google.golang.org/genproto/googleapis/api/expr/v1alpha1"
 )
 
@@ -112,7 +111,7 @@ func extractCELFromMessage(file *file, msgValue ir.MessageValue, irSym ir.Symbol
 				continue
 			}
 
-			// Handle nested message field (cel.expression)
+			// Handle nested Rule message (cel field containing Rule messages with expressions)
 			if fieldNumber == celFieldNumberInFieldRules || fieldNumber == celFieldNumberInMessageRules {
 				// This is a Rule message, look for the expression field
 				nestedMsg := element.AsMessage()
@@ -134,6 +133,7 @@ func extractCELFromMessage(file *file, msgValue ir.MessageValue, irSym ir.Symbol
 					// Recursively check nested messages
 					results = append(results, extractCELFromMessage(file, nestedMsg, irSym)...)
 				}
+				continue
 			}
 
 			// Handle cel_expression string field
@@ -144,7 +144,17 @@ func extractCELFromMessage(file *file, msgValue ir.MessageValue, irSym ir.Symbol
 						span:       element.AST().Span(),
 						irMember:   getMember(),
 					})
+					continue
 				}
+				// Not a string value (e.g. a nested FieldRules message at the same field number
+				// as a MessageRules cel_expression field) - fall through to general recursion.
+			}
+
+			// General case: recursively search any nested message for CEL expressions.
+			// This handles nested structures like repeated.items.cel, map.keys.cel,
+			// map.values.cel, and any other type-specific rule nesting in protovalidate.
+			if nestedMsg := element.AsMessage(); !nestedMsg.IsZero() {
+				results = append(results, extractCELFromMessage(file, nestedMsg, irSym)...)
 			}
 		}
 	}
@@ -192,68 +202,6 @@ func collectCELTokens(
 	collectMacroTokens(sourceInfo, exprInfo.span, exprInfo.expression, collectToken)
 }
 
-// createCELSpan creates a source.Span for a CEL token given its start and end offsets within the CEL expression.
-// The exprLiteralSpan is the span of the string literal containing the CEL expression (including quotes).
-func createCELSpan(celStart, celEnd int32, exprLiteralSpan source.Span) source.Span {
-	// Check if this is a multi-line span (covers multiple string literals)
-	startLoc := exprLiteralSpan.StartLoc()
-	endLoc := exprLiteralSpan.EndLoc()
-	if startLoc.Line != endLoc.Line {
-		// Multi-line span - use special handling for concatenated literals
-		return createCELSpanMultiline(celStart, celEnd, exprLiteralSpan)
-	}
-
-	// For single-line literals, use simple offset calculation
-	literalText := exprLiteralSpan.Text()
-	if len(literalText) < 2 {
-		return source.Span{}
-	}
-
-	// Calculate offset from start of file
-	// exprLiteralSpan.Start is the byte offset of the opening quote
-	// Add 1 for the quote, then add the CEL offset
-	fileStart := exprLiteralSpan.Start + 1 + int(celStart)
-	fileEnd := exprLiteralSpan.Start + 1 + int(celEnd)
-
-	// Validate bounds
-	if fileEnd > exprLiteralSpan.End {
-		return source.Span{}
-	}
-
-	return source.Span{File: exprLiteralSpan.File, Start: fileStart, End: fileEnd}
-}
-
-// createCELSpanMultiline handles CEL token spans for multi-line expressions.
-// Multi-line spans contain multiple quoted strings like: "first" "second"
-// CEL concatenates them into: "firstsecond"
-// This function maps CEL offsets back to file positions.
-func createCELSpanMultiline(celStart, celEnd int32, multilineSpan source.Span) source.Span {
-	spanText := multilineSpan.Text()
-	celPos := 0 // Current position in concatenated CEL string
-
-	// Walk through the span text, tracking both file position and CEL position
-	for i := 0; i < len(spanText); i++ {
-		if spanText[i] != '"' {
-			continue
-		}
-
-		// Found opening quote - scan the string content
-		i++
-		for i < len(spanText) && spanText[i] != '"' {
-			// Check if we've found the token start
-			if celPos == int(celStart) {
-				fileStart := multilineSpan.Start + i
-				fileEnd := fileStart + int(celEnd-celStart)
-				return source.Span{File: multilineSpan.File, Start: fileStart, End: fileEnd}
-			}
-			celPos++
-			i++
-		}
-	}
-
-	return source.Span{}
-}
-
 // collectMacroTokens processes CEL macro calls to highlight macro function names.
 // Macros like has(), all(), exists(), map(), filter() are expanded in the main AST,
 // but CEL preserves the original macro calls in sourceInfo.MacroCalls.
@@ -278,8 +226,8 @@ func collectMacroTokens(
 			continue
 		}
 
-		// Get the position of the macro call
-		celOffset, ok := sourceInfo.Positions[macroID]
+		// Get the position of the macro call (rune offset from CEL)
+		celRuneOffset, ok := sourceInfo.Positions[macroID]
 		if !ok {
 			continue
 		}
@@ -289,19 +237,21 @@ func collectMacroTokens(
 		if callExpr.CallExpr.Target != nil {
 			// Method call - search for ".funcName" after the target
 			targetID := callExpr.CallExpr.Target.Id
-			if targetOffset, ok := sourceInfo.Positions[targetID]; ok {
-				tokenSpan := findNameAfterDot(targetOffset, funcName, exprString, exprLiteralSpan)
+			if targetRuneOffset, ok := sourceInfo.Positions[targetID]; ok {
+				targetByteOffset := celRuneOffsetToByteOffset(exprString, targetRuneOffset)
+				tokenSpan := findNameAfterDot(targetByteOffset, funcName, exprString, exprLiteralSpan)
 				if !tokenSpan.IsZero() {
 					collectToken(tokenSpan, semanticTypeMacro, 0, keyword.Unknown)
 				}
 			}
 		} else {
 			// Standalone function call - CEL position points to opening paren, look backwards
-			funcStart := int(celOffset) - len(funcName)
+			celByteOffset := celRuneOffsetToByteOffset(exprString, celRuneOffset)
+			funcStart := celByteOffset - len(funcName)
 			funcEnd := funcStart + len(funcName)
 			if funcStart >= 0 && funcEnd <= len(exprString) {
 				if exprString[funcStart:funcEnd] == funcName {
-					tokenSpan := createCELSpan(int32(funcStart), int32(funcEnd), exprLiteralSpan)
+					tokenSpan := createCELSpan(funcStart, funcEnd, exprLiteralSpan)
 					if !tokenSpan.IsZero() {
 						collectToken(tokenSpan, semanticTypeMacro, 0, keyword.Unknown)
 					}
@@ -326,11 +276,13 @@ func walkCELExprWithVars(
 		return
 	}
 
-	// Get the byte offset for this expression within the CEL string
-	celOffset, ok := sourceInfo.Positions[expr.Id]
+	// Get the CEL rune offset for this expression and convert to byte offset.
+	// CEL tracks positions as Unicode code point (rune) offsets, not byte offsets.
+	celRuneOffset, ok := sourceInfo.Positions[expr.Id]
 	if !ok {
 		return
 	}
+	celByteOffset := celRuneOffsetToByteOffset(exprString, celRuneOffset)
 
 	var tokenSpan source.Span
 
@@ -356,7 +308,8 @@ func walkCELExprWithVars(
 		}
 
 		if offsetRange, ok := offsetRanges[expr.Id]; ok {
-			tokenSpan = createCELSpan(offsetRange.Start, offsetRange.Stop, exprLiteralSpan)
+			byteStart, byteStop := celOffsetRangeToByteRange(exprString, offsetRange)
+			tokenSpan = createCELSpan(byteStart, byteStop, exprLiteralSpan)
 			if !tokenSpan.IsZero() {
 				collectToken(tokenSpan, tokenType, tokenModifier, keyword.Unknown)
 			}
@@ -373,8 +326,9 @@ func walkCELExprWithVars(
 
 		// Highlight the field name
 		if sel.Operand != nil {
-			if targetOffset, ok := sourceInfo.Positions[sel.Operand.Id]; ok {
-				tokenSpan = findNameAfterDot(targetOffset, sel.Field, exprString, exprLiteralSpan)
+			if targetRuneOffset, ok := sourceInfo.Positions[sel.Operand.Id]; ok {
+				targetByteOffset := celRuneOffsetToByteOffset(exprString, targetRuneOffset)
+				tokenSpan = findNameAfterDot(targetByteOffset, sel.Field, exprString, exprLiteralSpan)
 				if !tokenSpan.IsZero() {
 					collectToken(tokenSpan, semanticTypeProperty, 0, keyword.Unknown)
 				}
@@ -397,7 +351,8 @@ func walkCELExprWithVars(
 		if _, isOperator := celOperatorSymbol(funcName); isOperator {
 			// This is an operator - use offset ranges from CEL's native AST
 			if offsetRange, ok := offsetRanges[expr.Id]; ok {
-				tokenSpan = createCELSpan(offsetRange.Start, offsetRange.Stop, exprLiteralSpan)
+				byteStart, byteStop := celOffsetRangeToByteRange(exprString, offsetRange)
+				tokenSpan = createCELSpan(byteStart, byteStop, exprLiteralSpan)
 				if !tokenSpan.IsZero() {
 					collectToken(tokenSpan, semanticTypeOperator, 0, keyword.Unknown)
 				}
@@ -411,7 +366,7 @@ func walkCELExprWithVars(
 			if isCELMacroFunction(funcName) {
 				// Macro functions (has, all, exists, map, filter)
 				tokenType = semanticTypeMacro
-			} else if isCELBuiltinTypeFunction(funcName) {
+			} else if overloads.IsTypeConversionFunction(funcName) {
 				// Built-in type conversion functions (int, uint, string, etc.)
 				tokenType = semanticTypeType
 				tokenModifier = semanticModifierDefaultLibrary
@@ -425,8 +380,9 @@ func walkCELExprWithVars(
 
 			if call.Target != nil {
 				// Method call - search for the function name after the target
-				if targetOffset, ok := sourceInfo.Positions[call.Target.Id]; ok {
-					tokenSpan = findNameAfterDot(targetOffset, funcName, exprString, exprLiteralSpan)
+				if targetRuneOffset, ok := sourceInfo.Positions[call.Target.Id]; ok {
+					targetByteOffset := celRuneOffsetToByteOffset(exprString, targetRuneOffset)
+					tokenSpan = findNameAfterDot(targetByteOffset, funcName, exprString, exprLiteralSpan)
 					if !tokenSpan.IsZero() {
 						collectToken(tokenSpan, tokenType, tokenModifier, keyword.Unknown)
 					}
@@ -434,11 +390,11 @@ func walkCELExprWithVars(
 			} else {
 				// Standalone function call (no target)
 				// CEL's position typically points to the opening paren, so look backwards for the function name
-				funcStart := int(celOffset) - len(funcName)
+				funcStart := celByteOffset - len(funcName)
 				funcEnd := funcStart + len(funcName)
 				if funcStart >= 0 && funcEnd <= len(exprString) {
 					if exprString[funcStart:funcEnd] == funcName {
-						tokenSpan = createCELSpan(int32(funcStart), int32(funcEnd), exprLiteralSpan)
+						tokenSpan = createCELSpan(funcStart, funcEnd, exprLiteralSpan)
 						if !tokenSpan.IsZero() {
 							collectToken(tokenSpan, tokenType, tokenModifier, keyword.Unknown)
 						}
@@ -460,7 +416,8 @@ func walkCELExprWithVars(
 		case *exprpb.Constant_StringValue:
 			// String literal - use offset ranges from CEL's native AST
 			if offsetRange, ok := offsetRanges[expr.Id]; ok {
-				tokenSpan = createCELSpan(offsetRange.Start, offsetRange.Stop, exprLiteralSpan)
+				byteStart, byteStop := celOffsetRangeToByteRange(exprString, offsetRange)
+				tokenSpan = createCELSpan(byteStart, byteStop, exprLiteralSpan)
 				if !tokenSpan.IsZero() {
 					collectToken(tokenSpan, semanticTypeString, 0, keyword.Unknown)
 				}
@@ -469,7 +426,8 @@ func walkCELExprWithVars(
 		case *exprpb.Constant_Int64Value, *exprpb.Constant_Uint64Value, *exprpb.Constant_DoubleValue:
 			// Number literal - use offset ranges from CEL's native AST
 			if offsetRange, ok := offsetRanges[expr.Id]; ok {
-				tokenSpan = createCELSpan(offsetRange.Start, offsetRange.Stop, exprLiteralSpan)
+				byteStart, byteStop := celOffsetRangeToByteRange(exprString, offsetRange)
+				tokenSpan = createCELSpan(byteStart, byteStop, exprLiteralSpan)
 				if !tokenSpan.IsZero() {
 					collectToken(tokenSpan, semanticTypeNumber, 0, keyword.Unknown)
 				}
@@ -478,7 +436,8 @@ func walkCELExprWithVars(
 		case *exprpb.Constant_BoolValue:
 			// Boolean literal - use offset ranges from CEL's native AST
 			if offsetRange, ok := offsetRanges[expr.Id]; ok {
-				tokenSpan = createCELSpan(offsetRange.Start, offsetRange.Stop, exprLiteralSpan)
+				byteStart, byteStop := celOffsetRangeToByteRange(exprString, offsetRange)
+				tokenSpan = createCELSpan(byteStart, byteStop, exprLiteralSpan)
 				if !tokenSpan.IsZero() {
 					collectToken(tokenSpan, semanticTypeKeyword, 0, keyword.Unknown)
 				}
@@ -548,85 +507,3 @@ func walkCELExprWithVars(
 		}
 	}
 }
-
-// isCELKeyword returns true if the identifier is a CEL reserved keyword.
-// See https://github.com/google/cel-spec/blob/master/doc/langdef.md#syntax
-func isCELKeyword(name string) bool {
-	keywords := map[string]bool{
-		// Literals
-		"true":  true,
-		"false": true,
-		"null":  true,
-		// Special identifiers
-		"this": true,
-	}
-	return keywords[name]
-}
-
-// isCELBuiltinTypeFunction returns true if the function name is a CEL built-in type conversion function.
-// See https://github.com/google/cel-spec/blob/master/doc/langdef.md#gradual-type-checking
-func isCELBuiltinTypeFunction(funcName string) bool {
-	builtins := map[string]bool{
-		"int":       true,
-		"uint":      true,
-		"double":    true,
-		"bool":      true,
-		"string":    true,
-		"bytes":     true,
-		"duration":  true,
-		"timestamp": true,
-		"dyn":       true,
-		"type":      true,
-	}
-	return builtins[funcName]
-}
-
-// isCELMacroFunction returns true if the function name is a CEL macro (comprehension or special function).
-// See https://github.com/google/cel-spec/blob/master/doc/langdef.md#macros
-func isCELMacroFunction(funcName string) bool {
-	return funcName == operators.Has ||
-		funcName == operators.All ||
-		funcName == operators.Exists ||
-		funcName == operators.ExistsOne ||
-		funcName == operators.Map ||
-		funcName == operators.Filter
-	// Note: "size" is intentionally excluded as it's more commonly used as a method
-}
-
-// celOperatorSymbol maps CEL operator function names to their operator symbols.
-// CEL represents operators as function calls with names like _&&_, _||_, _>_, etc.
-// Returns the operator symbol and true if the function name represents an operator.
-// See https://github.com/google/cel-spec/blob/master/doc/langdef.md#operators
-func celOperatorSymbol(funcName string) (string, bool) {
-	// Use cel-go's operators.FindReverse to get the display symbol
-	if symbol, found := operators.FindReverse(funcName); found {
-		return symbol, true
-	}
-
-	// Special case: ternary operator doesn't have a text representation in FindReverse
-	// We highlight the '?' part of the ternary operator
-	if funcName == operators.Conditional {
-		return "?", true
-	}
-
-	return "", false
-}
-
-// findNameAfterDot searches for ".name" after targetOffset and returns the span of just the name (without the dot).
-// Returns zero span if not found.
-func findNameAfterDot(
-	targetOffset int32,
-	name string,
-	exprString string,
-	exprLiteralSpan source.Span,
-) source.Span {
-	searchStart := int(targetOffset)
-	searchRegion := exprString[searchStart:]
-
-	if idx := strings.Index(searchRegion, "."+name); idx >= 0 {
-		nameStart := searchStart + idx + 1 // +1 to skip the dot
-		nameEnd := nameStart + len(name)
-		return createCELSpan(int32(nameStart), int32(nameEnd), exprLiteralSpan)
-	}
-	return source.Span{}
-}
