@@ -139,6 +139,7 @@ func (s *server) Initialize(
 				CodeActionKinds: []protocol.CodeActionKind{
 					protocol.SourceOrganizeImports,
 					protocol.RefactorRewrite,
+					protocol.QuickFix,
 				},
 			},
 			CompletionProvider: &protocol.CompletionOptions{
@@ -192,6 +193,12 @@ func (s *server) SetTrace(
 // The client will wait until Shutdown returns, and then call Exit.
 func (s *server) Shutdown(ctx context.Context) error {
 	s.lsp.shutdown = true
+	// Cancel any in-progress checks for all tracked files, then cancel the
+	// connection context to stop any remaining background goroutines.
+	for _, file := range s.lsp.fileManager.uriToFile.Range {
+		file.CancelChecks(ctx)
+	}
+	s.lsp.connCancel()
 	return nil
 }
 
@@ -340,7 +347,19 @@ func (s *server) Hover(
 	ctx context.Context,
 	params *protocol.HoverParams,
 ) (*protocol.Hover, error) {
-	symbol := s.getSymbol(ctx, params.TextDocument.URI, params.Position)
+	file := s.fileManager.Get(params.TextDocument.URI)
+	if file == nil {
+		return nil, nil
+	}
+
+	// First, try to get CEL hover documentation
+	celHover := getCELHover(file, params.Position, s.celEnv)
+	if celHover != nil {
+		return celHover, nil
+	}
+
+	// Fall back to regular symbol hover
+	symbol := file.SymbolAt(ctx, params.Position)
 	if symbol == nil {
 		return nil, nil
 	}
@@ -512,6 +531,10 @@ func (s *server) CodeAction(ctx context.Context, params *protocol.CodeActionPara
 			actions = append(actions, *deprecateAction)
 		}
 	}
+	if _, ok := codeActionSet[protocol.QuickFix]; len(codeActionSet) == 0 || ok {
+		lintIgnoreActions := s.getLintIgnoreCodeActions(ctx, file, params)
+		actions = append(actions, lintIgnoreActions...)
+	}
 	return actions, nil
 }
 
