@@ -514,7 +514,7 @@ type policyV1Beta1PolicyConfig_BreakingConfig struct {
 
 // policyV1Beta1PolicyConfig_PluginConfig is a stable JSON representation of the buf.registry.policy.v1beta1.PolicyConfig.PluginConfig.
 type policyV1Beta1PolicyConfig_PluginConfig struct {
-	Name    policyV1Beta1PolicyConfig_PluginConfig_Name `json:"name,omitempty"`
+	Name    policyV1Beta1PolicyConfig_PluginConfig_Name `json:"name"`
 	Options []*optionV1Option                           `json:"options,omitempty"`
 	Args    []string                                    `json:"args,omitempty"`
 }
