@@ -64,7 +64,16 @@ func New(options ...ValidatorOption) (Validator, error) {
 		opt.applyToValidator(&cfg)
 	}
 
-	env, err := cel.NewEnv(cel.Lib(pvcel.NewLibrary()))
+	reg, err := newRegistry()
+	if err != nil {
+		return nil, fmt.Errorf(
+			"failed to construct CEL type registry: %w", err)
+	}
+	env, err := cel.NewEnv(
+		cel.CustomTypeProvider(reg),
+		cel.CustomTypeAdapter(reg),
+		cel.Lib(pvcel.NewLibrary()),
+	)
 	if err != nil {
 		return nil, fmt.Errorf(
 			"failed to construct CEL environment: %w", err)
